package jp.veritrans.tercerog.sample.util;

import java.io.IOException;
import java.io.Writer;

public class CSVWriter {

	protected Writer out;

	// 引用符の設定
	private static final boolean ALWAYS_QUOTE_DEFAULT = false;
	protected boolean alwaysQuote = ALWAYS_QUOTE_DEFAULT;
	private static final char QUOTE_DEFAULT = '"';
	protected char quoteChar = QUOTE_DEFAULT;

	// 区切り子文字
	private static final char DELIMITER_DEFAULT = ',';
	protected char delimiterChar = DELIMITER_DEFAULT;

	// 行区切り文字
	private static final String LINE_ENDING_DEFAULT = "\n";
	protected String lineEnding = LINE_ENDING_DEFAULT;

	// Flushの設定
	private static final boolean AUTO_FLUSH_DEFAULT = true;
	protected boolean autoFlush = AUTO_FLUSH_DEFAULT;

	protected boolean newLine = true;

	/**
	 * Create printer to print values to the given stream.
	 * 
	 * @param out Stream to which to print.
	 * @param alwaysQuote true if quotes should be used.
	 * @param autoFlush true if auto flushing be enabled.
	 * @throws Exception
	 */
	public CSVWriter(Writer out, boolean alwaysQuote, boolean autoFlush) throws Exception {
		this.out = out;
		setAlwaysQuote(alwaysQuote);
		setAutoFlush(autoFlush);
	}

	/**
	 * Set whether values should always be quoted or not.
	 * 
	 * @param alwaysQuote
	 */
	private void setAlwaysQuote(boolean alwaysQuote) {
		this.alwaysQuote = alwaysQuote;
	}

	/**
	 * Set whether flushing should be enabled or not.
	 * 
	 * @param autoFlush
	 */
	private void setAutoFlush(boolean autoFlush) {
		this.autoFlush = autoFlush;
	}

	/**
	 * Writes a single line of comma separated values.
	 * 
	 * @param values values to be outputted.
	 * @throws IOException If an error occurs while writing the line.
	 */
	public void writeln(String[] values) throws IOException {
		try {
			write(values);
			writeln();
		} catch (IOException iox) {
			throw iox;
		}
	}

	/**
	 * Writes a single line of comma separated values.
	 * 
	 * @param values Values to be outputted.
	 * @throws IOException If an error occurs while writing the line.
	 */
	private void write(String[] values) throws IOException {
		try {
			for (int i = 0; i < values.length; i++) {
				write(values[i]);
			}
		} catch (IOException iox) {
			throw iox;
		}
	}

	/**
	 * Write the string as the next value on the line.
	 * 
	 * @param value value to be outputted.
	 * @throws IOException If an error occurs while writing the value.
	 */
	private void write(String value) throws IOException {
		try {
			if (value == null)
				value = "";
			boolean quote = false;
			if (alwaysQuote) {
				quote = true;
			} else if (value.length() > 0) {
				char c = value.charAt(0);
				if (newLine && (c < '0' || (c > '9' && c < 'A') || (c > 'Z' && c < 'a') || (c > 'z'))) {
					quote = true;
				}
				if (c == ' ' || c == '\f' || c == '\t') {
					quote = true;
				}
				for (int i = 0; i < value.length(); i++) {
					c = value.charAt(i);
					if (c == quoteChar || c == delimiterChar || c == '\n' || c == '\r') {
						quote = true;
					}
				}
				if (c == ' ' || c == '\f' || c == '\t') {
					quote = true;
				}
			} else if (newLine) {
				quote = true;
			}
			if (newLine) {
				newLine = false;
			} else {
				out.write(delimiterChar);
			}
			if (quote) {
				out.write(escapeAndQuote(value));
			} else {
				out.write(value);
			}
			if (autoFlush)
				flush();
		} catch (IOException iox) {
			throw iox;
		}
	}

	/**
	 * Output a blank line.
	 * 
	 * @throws IOException
	 */
	private void writeln() throws IOException {
		try {
			out.write(lineEnding);
			if (autoFlush)
				flush();
			newLine = true;
		} catch (IOException iox) {
			throw iox;
		}
	}

	/**
	 * Enclose the value in quotes and escape the quote and comma characters.
	 * 
	 * @param value Value to be written.
	 * @return escaped and quoted value.
	 */
	private String escapeAndQuote(String value) {
		int count = 2;
		for (int i = 0; i < value.length(); i++) {
			char c = value.charAt(i);
			switch (c) {
			case '\n':
			case '\r':
			case '\\': {
				count++;
			}
				break;
			default: {
				if (c == quoteChar) {
					count++;
				}
			}
				break;
			}
		}
		StringBuffer sb = new StringBuffer(value.length() + count);
		sb.append(quoteChar);
		for (int i = 0; i < value.length(); i++) {
			char c = value.charAt(i);
			switch (c) {
			case '\n': {
				sb.append("\\n");
			}
				break;
			case '\r': {
				sb.append("\\r");
			}
				break;
			case '\\': {
				sb.append("\\\\");
			}
				break;
			default: {
				if (c == quoteChar) {
					sb.append("\\" + quoteChar);
				} else {
					sb.append(c);
				}
			}
			}
		}
		sb.append(quoteChar);
		return (sb.toString());
	}

	/**
	 * Flush any data written to underlying streams.
	 * 
	 * @throws IOException If any error occurs during flush.
	 */
	private void flush() throws IOException {
		out.flush();
	}

}
