package jp.veritrans.tercerog.sample.web.amazonpay;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import jp.veritrans.tercerog.mdk.ITransaction;
import jp.veritrans.tercerog.mdk.TransactionFactory;
import jp.veritrans.tercerog.mdk.dto.AmazonpayGetAddressRequestDto;
import jp.veritrans.tercerog.mdk.dto.AmazonpayGetAddressResponseDto;
import jp.veritrans.tercerog.sample.web.MdkSampleBaseServlet;

/**
 * Amazon Pay決済確認画面の初期表示クラス。
 */
public class AuthorizeConfirm extends MdkSampleBaseServlet {
    private static final long serialVersionUID = 1L;
    // デフォルトの文字コード
    private static final String DEFAULT_CHARSET = "UTF-8";
    // デフォルトのアルゴリズム
    private static final String CHECKSUM_ALGORITHM = "SHA-1";

    /**
     * @see MdkSampleBaseServlet#doProcess
     */
    @Override
    protected void doProcess(final HttpServletRequest request, final HttpServletResponse response)
            throws ServletException, IOException {

        // エラー情報の初期化
        String warning = null;

        // 3GPSMDK.propertiesファイルから必要な値を取得。
        Locale objLcl = request.getLocale();
        ResourceBundle rb = ResourceBundle.getBundle("3GPSMDK", objLcl);

        // マーチャントCCIDの取得
        String merchantCcId = rb.getString("MERCHANT_CCID");
        // マーチャントパスワードの取得
        String merchantPw = rb.getString("MERCHANT_SECRET_KEY");

        // ハッシュ値による改竄チェック
        String orderId = request.getParameter("orderId");
        String orderHash = request.getParameter("orderHash");

        try {
            // 取引ID、マーチャントCCID、マーチャントパスワード、文字コードを用いて改竄チェックを行う。
            boolean checkResult = checkOrderHash(merchantCcId, merchantPw, orderId, orderHash);
            if (!checkResult) {
                warning = "<font color='#ff0000'><b>【パラメータ改竄】パラメータ情報が改竄されています。</b></font>";
                request.setAttribute("warning", warning);

                // 結果画面の表示
                forward(request, response, "./AuthorizeConfirm.jsp");
                return;
            }

            //消費者の住所等を取得
            AmazonpayGetAddressRequestDto reqDto = new AmazonpayGetAddressRequestDto();
            reqDto.setOrderId(request.getParameter("orderId"));
            reqDto.setAddInfoRespFlag("1");

            ITransaction tran = TransactionFactory.getInstance(reqDto);
            AmazonpayGetAddressResponseDto resDto = (AmazonpayGetAddressResponseDto) tran.execute();

            if (resDto.getMstatus().equals("success")) {
                // 取得した住所から送料・決済金額を計算して更新して決済確認画面に反映する
                HttpSession session = request.getSession(true);
                int originalAmount = (Integer) session.getAttribute("originalAmount");
                                
                int shippingAmount = 0;
                int amount = originalAmount + shippingAmount;

                // サーバー側で保持する情報
                session.setAttribute("amount", amount);
                session.setAttribute("payRedirectUrl", resDto.getPayRedirectUrl());

                // 画面に表示するパラメータ
                request.setAttribute("shippingName", resDto.getShippingName());
                request.setAttribute("shippingPostalCode", resDto.getShippingPostalCode());
                request.setAttribute("shippingAddress1", resDto.getShippingAddress1());
                request.setAttribute("shippingAddress2", resDto.getShippingAddress2());
                request.setAttribute("shippingAddress3", resDto.getShippingAddress3());
                request.setAttribute("shippingPhone", resDto.getShippingPhone());

                request.setAttribute("buyerId", resDto.getBuyerId());
                request.setAttribute("buyerName", resDto.getBuyerName());
                request.setAttribute("billingAddress1", resDto.getBillingAddress1());
                request.setAttribute("billingAddress2", resDto.getBillingAddress2());
                request.setAttribute("billingAddress3", resDto.getBillingAddress3());
                request.setAttribute("buyerEmail", resDto.getBuyerEmail());
                request.setAttribute("paymentPreferences", resDto.getPaymentPreferences());
                request.setAttribute("amount", amount);
                request.setAttribute("shippingAmount", shippingAmount);

                // 決済確認画面の表示 ※ダミー取引か本番取引で画面を変える
                if ("1".equals(rb.getString("DUMMY_REQUEST"))) {
                    // checkoutResultReturnUrlはダミーAmazonPayの画面に遷移する際に必要
                    String payConfirmScreenUrl = request.getRequestURL().toString();
                    String checkoutResultReturnUrl = payConfirmScreenUrl + "?orderId=" + orderId + "&orderHash=" + orderHash;
                    request.setAttribute("checkoutResultReturnUrl", checkoutResultReturnUrl);

                    forward(request, response, "./AuthorizeConfirmDummy.jsp");
                    return;
                } else {
                    // checkoutSessionIdは住所変更などでAmazonPay側の画面に遷移する際に必要
                    request.setAttribute("checkoutSessionId", resDto.getCheckoutSessionId());

                    forward(request, response, "./AuthorizeConfirm.jsp");
                    return;
                }

            } else {
                // 結果画面を表示します
                request.setAttribute("orderId", resDto.getOrderId());
                request.setAttribute("mStatus", resDto.getMstatus());
                request.setAttribute("mErrMsg", resDto.getMerrMsg());
                request.setAttribute("vResultCode", resDto.getVResultCode());

                forward(request, response, "./AuthorizeExec.jsp");
                return;
            }
        } catch (Exception ex) {
            // トランザクションオブジェクトの生成失敗など、Exceptionが発生した場合はエラー画面を表示します。
            System.out.println("Exception Occured !");
            System.out.println("Exception Message: " + ex.getMessage());
            ex.printStackTrace();
            request.setAttribute("mErrMsg", "Exception Occured : " + ex.getMessage());
            forward(request, response, "./AuthorizeExec.jsp");
            return;
        }
    }

    /**
     * ハッシュ値の算出およびチェックを行う。
     *
     * @param merchantCcId マーチャントCCID
     * @param merchantPw マーチャントパスワード
     * @param orderId 取引ID
     * @param orderHash ハッシュ値
     * @return ハッシュ値の整合性確認結果
     * @throws NoSuchAlgorithmException
     * @throws UnsupportedEncodingException
     */
    private boolean checkOrderHash(final String merchantCcId, final String merchantPw, final String orderId,
            final String orderHash) throws NoSuchAlgorithmException, UnsupportedEncodingException {

        String hashBaseString = merchantCcId + orderId + merchantPw;

        // ハッシュ値への変換
        MessageDigest md = MessageDigest.getInstance(CHECKSUM_ALGORITHM);
        md.update(hashBaseString.getBytes(DEFAULT_CHARSET));
        byte[] hashBytes = md.digest();

        // バイト配列から16進文字列への変換
        StringBuilder sb = new StringBuilder();
        for (byte b : hashBytes) {
            String hex = String.format("%02x", b);
            sb.append(hex);
        }
        String hash = sb.toString();

        return orderHash.contentEquals(hash);
    }

}
